# Supports Organizer.rb
=begin rdoc
= Loose Geometry to Groups
Copyright 2009, Chris Fullmer
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

== Information
Author:: Chris Fullmer
Organization:: www.ChrisFullmer.com and distributed on www.smustard.com
Name:: Loose Geometry to Groups
Version:: v1.2
SU Version:: Developed on SU 7, should be compatible with 6 and 5 at least.
Date:: June 15, 2009
Description:: This script groups all loose geometry into multiple groups.
Usage:: Run the script from Plugins>Loose Geometry to Groups.  If the user has geometry selected, it will work the selection.  If nothing is slected, it will work with all loose geometry.  It will not group components or existing groups.  It makes a new group for all connected geometry.  So if there are 20 separate boxes, it will make 20 separate groups - 1 for each box.  I use this to quickly group large amounts of quickly made 3d buildings over bery large areas of land.  This way I do not have so much loose geometry floating around.
History::
0.1:: 2009-02-07
   * First release
1.0:: 2009-02-13
   * Update rdoc compatiblity
   * New license info to conform to Smustards EULA
1.1:: 2009-03-09
   * Organizer.rb support. If you place this script into an organizer directory, it will appear in the correct submenu.  If its just in the plugins folder, it will appear in "Chris Fullmer Tools" submenu of the Plugins Menu."
1.2:: 2009-06-15
   * Fixed bug where it was not working with selections.
   * Fixed some internal coding that might make it slightly faster.   
=end

require 'sketchup.rb'

module Clf_loosegroups

	# Make groups of all geometry in the selection set or active_entities.
	def Clf_loosegroups.main
		now = Time.now
		model = Sketchup.active_model
		@entities = model.active_entities
		sel = model.selection
		@all_groups = []
		if sel[0]
			ents = sel.to_a
		else
			ents = @entities.to_a
		end
		
		if Sketchup.version_number < 7000000
			model.start_operation "Make Groups"
		else
			model.start_operation "Make Groups", true
		end
		
		chunk = 1000
		count = (ents.length/chunk)+1
		new_arr = []
		@top_level = ents[0].parent
		count.times do |e|
			place_number = (e*chunk)
			new_arr = ents[place_number,chunk]
			puts new_arr.length
			Clf_loosegroups.maker(new_arr)
		end
		
		model.commit_operation
		now2 = Time.now
		puts now2-now
	end

	def Clf_loosegroups.maker(ents)
		nowa = Time.now
		big_group = []
		while ents.length > 0
			if ((ents[0].is_a? Sketchup::Edge) || (ents[0].is_a? Sketchup::Face)) && ents[0].parent == @top_level
				to_group = ents[0].all_connected
				ents = ents - to_group
				group = @entities.add_group(to_group)
				#big_group << group
			else
				ents = ents - [ents[0]]
			end
		end
		#temp_all_groups = @entities.add_group(big_group)
		#@all_groups << temp_all_groups
		nowb = Time.now
		puts "---------"
		puts nowb-nowa
		puts "---------"
	end # maker
end




#----------Menu system----------
if( $submenu )
	organizer = $submenu 
	if !file_loaded?(__FILE__) then
		organizer.add_item("Loose Geometry to Groups") { Clf_loosegroups.main  }
	end
else
	if !file_loaded?('clf_menu_loader') then
		$clf_tools_menu = UI.menu("Plugins").add_submenu("Chris Fullmer Tools")
	end
	if !file_loaded?(__FILE__) then
		$clf_tools_menu.add_item("Loose Geometry to Groups") { Clf_loosegroups.main  }
	end
end

file_loaded('clf_menu_loader')
file_loaded(__FILE__)